/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.api.product.dto;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import org.hibernate.validator.constraints.Length;

import java.io.Serial;
import java.io.Serializable;

/**
 * 品牌店铺关联信息DTO
 *
 * @author FrozenWatermelon
 * @date 2021-05-08 13:31:45
 */
public class BrandShopDTO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    @Schema(description = "主键id")
    private Long brandShopId;

    @Schema(description = "店铺id")
    private Long shopId;

    @Schema(description = "品牌id")
    private Long brandId;

    @Schema(description = "授权资质图片，以,分割")
    @NotBlank(message = "授权资质图片不能为空")
    private String qualifications;

    @Schema(description = "类型 0：平台品牌，1：店铺自定义品牌")
    private Integer type;

    @Schema(description = "检索首字母")
    @Length(max = 1, message = "检索首字母长度不能超过1")
    private String firstLetter;

    @Schema(description = "排序")
    private Integer seq;

    @Schema(description = "是否置顶 0：不置顶  1：置顶")
    private Integer isTop;

    @Schema(description = "logo")
    private String imgUrl;

    @Schema(description = "品牌名称")
    private String name;

    @Schema(description = "品牌描述")
    private String desc;

    @Schema(description = "品牌状态")
    private Integer status;

    @Schema(description = "分类id")
    private Long categoryId;

    @Schema(description = "关联系统类型 1.店铺 2.平台端 3.供应商")
    private Integer sysType;

    public Integer getSysType() {
        return sysType;
    }

    public void setSysType(Integer sysType) {
        this.sysType = sysType;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public String getFirstLetter() {
        return firstLetter;
    }

    public void setFirstLetter(String firstLetter) {
        this.firstLetter = firstLetter;
    }

    public Integer getSeq() {
        return seq;
    }

    public void setSeq(Integer seq) {
        this.seq = seq;
    }

    public Integer getIsTop() {
        return isTop;
    }

    public void setIsTop(Integer isTop) {
        this.isTop = isTop;
    }

    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    public Long getBrandShopId() {
        return brandShopId;
    }

    public void setBrandShopId(Long brandShopId) {
        this.brandShopId = brandShopId;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public Long getBrandId() {
        return brandId;
    }

    public void setBrandId(Long brandId) {
        this.brandId = brandId;
    }

    public String getQualifications() {
        return qualifications;
    }

    public void setQualifications(String qualifications) {
        this.qualifications = qualifications;
    }

    public Integer getType() {
        return type;
    }

    public void setType(Integer type) {
        this.type = type;
    }

    public String getImgUrl() {
        return imgUrl;
    }

    public void setImgUrl(String imgUrl) {
        this.imgUrl = imgUrl;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }


    public Long getCategoryId() {
        return categoryId;
    }

    public void setCategoryId(Long categoryId) {
        this.categoryId = categoryId;
    }

    @Override
    public String toString() {
        return "BrandShopDTO{" +
                "brandShopId=" + brandShopId +
                ", shopId=" + shopId +
                ", brandId=" + brandId +
                ", qualifications='" + qualifications + '\'' +
                ", type=" + type +
                ", firstLetter='" + firstLetter + '\'' +
                ", seq=" + seq +
                ", isTop=" + isTop +
                ", imgUrl='" + imgUrl + '\'' +
                ", name='" + name + '\'' +
                ", categoryId=" + categoryId +
                ", desc='" + desc + '\'' +
                ", status=" + status +
                ", sysType=" + sysType +
                ", status=" + status +
                '}';
    }
}
