/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.api.product.vo;

import io.swagger.v3.oas.annotations.media.Schema;

import java.io.Serial;
import java.io.Serializable;

/**
 * @author FrozenWatermelon
 * @date 2020/12/22
 */
public class SkuStockVO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    @Schema(description = "skuId")
    private Long skuId;

    @Schema(description = "产品ID")
    private Long spuId;


    @Schema(description = "shopId")
    private Long shopId;

    @Schema(description = "orderId")
    private Long orderId;

    @Schema(description = "库存数量")
    private Integer stock;

    @Schema(description = "锁定库存数量")
    private Integer lockStock;

    @Schema(description = "销量")
    private Integer sale;

    @Schema(description = "sku和商品的状态，1.商品为上架且sku非禁用  0.商品非上架， 或者sku禁用")
    private Integer status;

    /**
     * 库存分片序号
     * 记录扣除分片的id，回调解锁的时候找到这个分片去扣锁定库存
     */
    @Schema(description = "库存分片序号")
    private Integer zoneNo;

    @Schema(description = "库存点id")
    private Long stockPointId;

    @Schema(description = "库存点类型 1仓库 2门店")
    private Integer stockPointType;

    @Schema(description = "操作类型 LuaOperateEnum")
    private Integer operateType;

    @Schema(description = "组合商品关联的主skuId")
    private Long mainSkuId;

    @Schema(description = "组合数量")
    private Integer comboCount;


    public SkuStockVO() {
    }

    public SkuStockVO(Long skuId) {
        this.skuId = skuId;
        this.stock = 0;
        this.lockStock = 0;
        this.sale = 0;
        this.stockPointId = 0L;
    }

    public SkuStockVO(Long skuId, Long stockPointId) {
        this.skuId = skuId;
        this.stockPointId = stockPointId;
        this.stock = 0;
        this.lockStock = 0;
        this.sale = 0;
    }

    public SkuStockVO(Long skuId, Integer stock, Long stockPointId, Integer stockPointType) {
        this.skuId = skuId;
        this.stock = stock;
        this.stockPointId = stockPointId;
        this.stockPointType = stockPointType;
        this.sale = 0;
    }

    public SkuStockVO(Long skuId, Integer stock, Integer lockStock, Integer sale, Long stockPointId) {
        this.skuId = skuId;
        this.stock = stock;
        this.lockStock = lockStock;
        this.sale = sale;
        this.stockPointId = stockPointId;
    }


    public SkuStockVO(Long skuId, Integer stock) {
        this.skuId = skuId;
        this.stock = stock;
        this.sale = 0;
    }
    public Long getMainSkuId() {
        return mainSkuId;
    }

    public void setMainSkuId(Long mainSkuId) {
        this.mainSkuId = mainSkuId;
    }

    public Integer getComboCount() {
        return comboCount;
    }

    public void setComboCount(Integer comboCount) {
        this.comboCount = comboCount;
    }

    public Long getStockPointId() {
        return stockPointId;
    }

    public void setStockPointId(Long stockPointId) {
        this.stockPointId = stockPointId;
    }

    public Integer getStockPointType() {
        return stockPointType;
    }

    public void setStockPointType(Integer stockPointType) {
        this.stockPointType = stockPointType;
    }

    public Long getSkuId() {
        return skuId;
    }

    public void setSkuId(Long skuId) {
        this.skuId = skuId;
    }

    public Long getSpuId() {
        return spuId;
    }

    public void setSpuId(Long spuId) {
        this.spuId = spuId;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public Long getOrderId() {
        return orderId;
    }

    public void setOrderId(Long orderId) {
        this.orderId = orderId;
    }

    public Integer getStock() {
        return stock;
    }

    public void setStock(Integer stock) {
        this.stock = stock;
    }

    public Integer getLockStock() {
        return lockStock;
    }

    public void setLockStock(Integer lockStock) {
        this.lockStock = lockStock;
    }

    public Integer getSale() {
        return sale;
    }

    public void setSale(Integer sale) {
        this.sale = sale;
    }

    public Integer getZoneNo() {
        return zoneNo;
    }

    public void setZoneNo(Integer zoneNo) {
        this.zoneNo = zoneNo;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Integer getOperateType() {
        return operateType;
    }

    public void setOperateType(Integer operateType) {
        this.operateType = operateType;
    }

    @Override
    public String toString() {
        return "SkuStockVO{" +
                "skuId=" + skuId +
                ", spuId=" + spuId +
                ", shopId=" + shopId +
                ", orderId=" + orderId +
                ", stock=" + stock +
                ", lockStock=" + lockStock +
                ", sale=" + sale +
                ", status=" + status +
                ", zoneNo=" + zoneNo +
                ", stockPointId=" + stockPointId +
                ", stockPointType=" + stockPointType +
                ", operateType=" + operateType +
                ", mainSkuId=" + mainSkuId +
                ", comboCount=" + comboCount +
                '}';
    }
}
