/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.rbac.mapper;

import com.tmerclub.cloud.api.rbac.bo.UriPermissionBO;
import com.tmerclub.cloud.rbac.dto.MenuPermissionDTO;
import com.tmerclub.cloud.rbac.model.MenuPermission;
import com.tmerclub.cloud.rbac.vo.MenuPermissionVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author FrozenWatermelon
 * @date 2020/09/03
 */
public interface MenuPermissionMapper {

    /**
     * 获取菜单资源列表
     *
     * @param menuPermissionDTO 搜索参数
     * @return 菜单资源列表
     */
    List<MenuPermissionVO> list(@Param("menuPermission") MenuPermissionDTO menuPermissionDTO);

    /**
     * 根据菜单资源id获取菜单资源
     *
     * @param menuPermissionId 菜单资源id
     * @return 菜单资源
     */
    MenuPermissionVO getByMenuPermissionId(@Param("menuPermissionId") Long menuPermissionId);

    /**
     * 保存菜单资源
     *
     * @param menuPermission 菜单资源
     */
    void save(@Param("menuPermission") MenuPermission menuPermission);

    /**
     * 更新菜单资源
     *
     * @param menuPermission 菜单资源
     */
    void update(@Param("menuPermission") MenuPermission menuPermission);

    /**
     * 根据菜单资源id删除菜单资源
     *
     * @param menuPermissionId
     * @param sysType
     */
    void deleteById(@Param("menuPermissionId") Long menuPermissionId, @Param("sysType") Integer sysType);

    /**
     * 获取某个类型用户的所有权限列表
     *
     * @param sysType 系统类型
     * @return 权限列表
     */
    List<String> listAllPermissionBySysType(@Param("sysType") Integer sysType);

    /**
     * 获取某个用户的权限列表
     *
     * @param userId  用户id
     * @param sysType 系统类型
     * @return 权限列表
     */
    List<String> listPermissionByUserIdAndSysType(@Param("userId") Long userId, @Param("sysType") Integer sysType);

    /**
     * 根据系统类型，获取该类型用户拥有的所有权限数据
     *
     * @param sysType 系统类型
     * @return uri权限列表
     */
    List<UriPermissionBO> listUriPermissionInfo(@Param("sysType") Integer sysType);

    /**
     * 根据menuId获取菜单资源列表
     *
     * @param menuId 菜单id
     * @return 菜单资源列表数据
     */
    List<MenuPermissionVO> listByMenuId(@Param("menuId") Long menuId);

    /**
     * 通过权限对应的编码获取权限信息
     *
     * @param permission 权限对应的编码
     * @param sysType    系统类型
     * @param menuPermissionId
     * @return
     */
    MenuPermission getByPermission(@Param("permission") String permission, @Param("sysType") Integer sysType, @Param("menuPermissionId") Long menuPermissionId);

    /**
     * 根据MenuPermIdLis获取菜单资源数据
     * @param menuPermIdList
     * @return
     */
    List<UriPermissionBO> getUriPermissionListByMenuPermIdList(@Param("menuPermIdList") List<Long> menuPermIdList);
}
