/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.delivery.mapper;

import com.tmerclub.cloud.api.delivery.vo.StationVO;
import com.tmerclub.cloud.common.database.util.PageAdapter;
import com.tmerclub.cloud.delivery.dto.OrderStationDTO;
import com.tmerclub.cloud.delivery.dto.StationDTO;
import com.tmerclub.cloud.delivery.model.Station;
import com.tmerclub.cloud.order.vo.StationDetailVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 自提点信息
 *
 * @author FrozenWatermelon
 * @date 2020-12-07 15:10:01
 */
public interface StationMapper {

    /**
     * 获取自提点信息列表
     *
     * @return 自提点信息列表
     */
    List<StationVO> list();

    /**
     * 获取自提点列表信息
     *
     * @param stationDTO
     * @return
     */
    List<StationVO> listStation(@Param("stationDTO") StationDTO stationDTO);

    /**
     * 根据自提点信息id获取自提点信息
     *
     * @param stationId 自提点信息id
     * @return 自提点信息
     */
    StationVO getByStationId(@Param("stationId") Long stationId);

    /**
     * 保存自提点信息
     *
     * @param station 自提点信息
     */
    void save(@Param("station") Station station);

    /**
     * 更新自提点信息
     *
     * @param station 自提点信息
     */
    void update(@Param("station") Station station);

    /**
     * 根据自提点信息id删除自提点信息
     *
     * @param stationId
     */
    void deleteById(@Param("stationId") Long stationId);


    /**
     * 条件查询，分页获取经纬度最近的自提门店信息列表
     *
     * @param orderStationDTO   自提门店筛选参数
     * @return 分页经纬度最近的自提门店信息列表
     */
    List<StationDetailVO> getStationList(@Param("page") PageAdapter page, @Param("orderStationDTO") OrderStationDTO orderStationDTO);


    /**
     * 获取自提点距离
     *
     * @param stationId 自提点id
     * @param lat       纬度
     * @param lng       经度
     * @return
     */
    Double getStationDistance(@Param("stationId") Long stationId, @Param("lat") Double lat, @Param("lng") Double lng);

    /**
     * 根据自提点id删除自提点（逻辑删除）
     *
     * @param stationId
     */
    void logicDeleteById(Long stationId);

    /**
     * 获取自提点列表
     * @param stationIds
     * @return
     */
    List<StationVO> getStationListById(@Param("stationIds") List<Long> stationIds);

    /**
     * 获取指点店铺的门店列表
     * @param shopId
     * @param status
     * @return
     */
    List<StationDetailVO> listSimpleStation(@Param("shopId") Long shopId, @Param("status") Integer status);
}
