/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.delivery.service;

import com.tmerclub.cloud.api.delivery.vo.DeliveryInfoVO;
import com.tmerclub.cloud.common.database.dto.PageDTO;
import com.tmerclub.cloud.common.database.vo.PageVO;
import com.tmerclub.cloud.delivery.dto.DeliveryCompanyDTO;
import com.tmerclub.cloud.delivery.model.DeliveryCompany;
import com.tmerclub.cloud.delivery.vo.DeliveryCompanyVO;

import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Set;

/**
 * 物流公司
 *
 * @author FrozenWatermelon
 * @date 2020-12-07 15:10:00
 */
public interface DeliveryCompanyService {

    /**
     * 获取物流公司列表
     *
     * @return 物流公司列表数据
     */
    List<DeliveryCompanyVO> list();

    /**
     * 根据物流公司id获取物流公司
     *
     * @param deliveryCompanyId 物流公司id
     * @return 物流公司
     */
    DeliveryCompanyVO getByDeliveryCompanyId(Long deliveryCompanyId);

    /**
     * 保存物流公司
     *
     * @param deliveryCompany 物流公司
     */
    void save(DeliveryCompany deliveryCompany);

    /**
     * 更新物流公司
     *
     * @param deliveryCompany 物流公司
     */
    void update(DeliveryCompany deliveryCompany);

    /**
     * 根据物流公司id删除物流公司
     *
     * @param deliveryCompanyId
     */
    void deleteById(Long deliveryCompanyId);

    /**
     * 分页获取物流公司列表数据
     *
     * @param pageDTO
     * @param deliveryCompanyDTO
     * @return
     */
    PageVO<DeliveryCompanyVO> page(PageDTO pageDTO, DeliveryCompanyDTO deliveryCompanyDTO);

    /**
     * 根据物流ID与物流单号查找物流信息
     *
     * @param delivery        快递公司信息
     * @param expNo           物流单号
     * @param consigneeMobile 收件人手机号
     * @return 物流信息
     * @throws UnsupportedEncodingException
     */
    DeliveryInfoVO query(DeliveryCompanyVO delivery, String expNo, String consigneeMobile);

    /**
     * 根据物流公司id列表，获取物流公司列表
     *
     * @param ids
     * @return
     */
    List<DeliveryCompanyVO> listByIds(Set<Long> ids);
}
