/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.order.service;

import com.tmerclub.cloud.common.database.dto.PageDTO;
import com.tmerclub.cloud.common.database.vo.PageVO;
import com.tmerclub.cloud.common.order.vo.OrderSelfStationVO;
import com.tmerclub.cloud.common.order.vo.OrderVO;
import com.tmerclub.cloud.order.model.OrderSelfStation;

import java.util.List;

/**
 * 自提订单自提点信息
 *
 * @author FrozenWatermelon
 * @date 2020-12-05 14:13:50
 */
public interface OrderSelfStationService {

    /**
     * 分页获取自提订单自提点信息列表
     *
     * @param pageDTO 分页参数
     * @return 自提订单自提点信息列表分页数据
     */
    PageVO<OrderSelfStationVO> page(PageDTO pageDTO);

    /**
     * 根据自提订单自提点信息id获取自提订单自提点信息
     *
     * @param id 自提订单自提点信息id
     * @return 自提订单自提点信息
     */
    OrderSelfStationVO getById(Long id);

    /**
     * 保存自提订单自提点信息
     *
     * @param orderSelfStation 自提订单自提点信息
     */
    void save(OrderSelfStation orderSelfStation);

    /**
     * 保存自提订单自提点信息
     *
     * @param orderSelfStations 自提订单自提点信息列表
     */
    void saveBatch(List<OrderSelfStation> orderSelfStations);

    /**
     * 更新自提订单自提点信息
     *
     * @param orderSelfStation 自提订单自提点信息
     */
    void update(OrderSelfStation orderSelfStation);

    /**
     * 根据自提订单自提点信息id删除自提订单自提点信息
     *
     * @param id
     */
    void deleteById(Long id);


    /**
     * 根据用户id获取用户自提订单信息
     *
     * @param userId 用户id
     * @return 用户自提订单信息
     */
    List<OrderSelfStation> getStationUserInfo(Long userId);

    /**
     * 根据自提点id和自提码获取订单信息
     *
     * @param stationId
     * @param code
     * @return
     */
    OrderVO getOrderByStationCode(Long stationId, String code);

    /**
     * 根据订单号获取用户自提订单信息
     *
     * @param orderSelfStation
     * @param orderId          订单号
     * @return 用户自提订单信息
     */
    OrderSelfStationVO getStationByOrderId(OrderSelfStationVO orderSelfStation, Long orderId);

    /**
     * 计算实时距离
     *
     * @param stationId
     * @param lat
     * @param lng
     * @return
     */
    Double setStationDistance(Long stationId, Double lat, Double lng);


    /**
     * 获取用户的常用自提信息
     *
     * @param userId 用户id
     * @return 用户的常用自提信息
     */
    List<OrderSelfStationVO> getUserStation(Long userId);

    /**
     * 根据用户id获取待核销的自提订单
     *
     * @param userId    用户id
     * @param stationId 自提点id
     * @param shopId    店铺id
     * @return
     */
    List<OrderSelfStationVO> getOrderSelfStationByStationIdOrShopId(Long userId, Long stationId, Long shopId);

    /**
     * 根据订单ids获取对应的自提信息
     *
     * @param orderIds
     * @return
     */
    List<OrderSelfStationVO> listSelfStationByOrderIds(List<Long> orderIds);
}
