/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.product.mapper;

import com.tmerclub.cloud.common.database.util.PageAdapter;
import com.tmerclub.cloud.common.product.dto.AttrLangDTO;
import com.tmerclub.cloud.common.product.vo.AttrVO;
import com.tmerclub.cloud.product.dto.AttrDTO;
import com.tmerclub.cloud.product.model.Attr;
import com.tmerclub.cloud.product.model.AttrLang;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 属性信息
 *
 * @author FrozenWatermelon
 * @date 2020-10-28 15:27:23
 */
public interface AttrMapper {

    /**
     * 获取属性信息列表
     *
     * @param pageAdapter 分页参数
     * @param attrDTO     属性数据
     * @return 属性信息列表
     */
    List<AttrVO> list(@Param("page") PageAdapter pageAdapter, @Param("attr") AttrDTO attrDTO);

    /**
     * 获取属性总数
     *
     * @param attrDTO
     * @return
     */
    Long countAttr(@Param("attr") AttrDTO attrDTO);

    /**
     * 根据属性信息id获取属性信息
     *
     * @param attrId 属性信息id
     * @return 属性信息
     */
    AttrVO getByAttrId(@Param("attrId") Long attrId);

    /**
     * 保存属性信息
     *
     * @param attr 属性信息
     */
    void save(@Param("attr") Attr attr);

    /**
     * 更新属性信息
     *
     * @param attr 属性信息
     */
    void update(@Param("attr") Attr attr);

    /**
     * 根据属性信息id删除属性信息
     *
     * @param attrId
     */
    void deleteById(@Param("attrId") Long attrId);

    /**
     * 根据分类和属性类型，获取对应的属性列表
     *
     * @param attrType
     * @param categoryId
     * @param shopId
     * @param sysType
     * @return
     */
    List<AttrVO> getAttrsByCategoryIdAndAttrType(@Param("attrType") Integer attrType, @Param("categoryId") Long categoryId, @Param("shopId") Long shopId, @Param("sysType") Integer sysType);


    /**
     * 统计属性名的数量
     *
     * @param attrLangList
     * @param shopId
     * @param attrType
     * @param attrId
     * @return
     */
    List<AttrLang> countAttrName(@Param("attrLangList") List<AttrLangDTO> attrLangList, @Param("shopId") Long shopId, @Param("attrType") Integer attrType, @Param("attrId") Long attrId);
}
