/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.product.service;

import com.tmerclub.cloud.common.database.dto.PageDTO;
import com.tmerclub.cloud.common.database.vo.PageVO;
import com.tmerclub.cloud.common.product.dto.SpuDTO;
import com.tmerclub.cloud.common.product.vo.GiveawayVO;
import com.tmerclub.cloud.product.dto.GiveawayDTO;
import com.tmerclub.cloud.product.model.Giveaway;

import java.util.List;

/**
 * 赠品表
 *
 * @author FrozenWatermelon
 * @date 2022-08-16 11:11:58
 */
public interface GiveawayService {

    /**
     * 分页获取赠品表列表
     *
     * @param pageDTO 分页参数
     * @return 赠品表列表分页数据
     */
    PageVO<Giveaway> page(PageDTO pageDTO);

    /**
     * 根据商品id获取赠品
     *
     * @param spuId 商品id
     * @return 赠品
     */
    GiveawayVO getBySpuId(Long spuId);

    /**
     * 根据店铺id获取赠品表列表
     *
     * @param pageDTO
     * @param giveawayDTO
     * @return
     */
    PageVO<GiveawayVO> listByShopId(PageDTO pageDTO, GiveawayDTO giveawayDTO);

    /**
     * 根据商品id获取所有赠品商品信息
     *
     * @param spuId
     * @return
     */
    GiveawayVO getGiveawayBySpuId(Long spuId);

    /**
     * 根据商品id获取所有赠品商品信息
     *
     * @param spuId
     * @return
     */
    GiveawayVO getGiveawaySpuBySpuId(Long spuId);

    /**
     * 根据赠品表id获取赠品表
     *
     * @param giveawayId 赠品表id
     * @return 赠品表
     */
    GiveawayVO getByGiveawayId(Long giveawayId);

    /**
     * 保存赠品表
     *
     * @param giveawayDTO 赠品表
     */
    void save(GiveawayDTO giveawayDTO);

    /**
     * 更新赠品表
     *
     * @param giveawayDTO 赠品表
     */
    void update(GiveawayDTO giveawayDTO);


    /**
     * 修改赠品状态
     *
     * @param giveawayId
     * @param status
     */
    void changeStatus(Long giveawayId, Integer status);



    /**
     * 获取应该结束但是没有结束的赠品列表
     *
     * @return 应该结束但是没有结束的赠品列表
     */
    List<Giveaway> listUnEndButNeedEndActivity();

    /**
     * 启动未开启的赠品活动
     */
    void startGiveawayActivity();

    /**
     * 改变商品类型，结束正在进行的赠品信息
     *
     * @param giveawayList 赠品活动列表
     */
    void changeSpuTypeByGiveawayActivityIdList(List<Giveaway> giveawayList);

    /**
     * 判断修改的商品售价会不会影响赠品活动
     *
     * @param spuDTO
     */
    void checkGiveawaySpuPrice(SpuDTO spuDTO);

    /**
     * 根据赠送商品id，获取主商品id列表
     *
     * @param spuId
     * @return
     */
    List<Long> listMainSpuIdByGiveawaySpuId(Long spuId);
}
