package com.tmerclub.cloud.search.bo;

import com.tmerclub.cloud.common.constant.Constant;
import com.tmerclub.cloud.search.canal.annotation.CanalModel;
import com.tmerclub.cloud.search.canal.common.FieldNamingPolicy;

import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * @author gaozijie
 * @since 2023-11-24
 */
@CanalModel(database = "mall4cloud" + Constant.UNDERLINE_ENV + "_order", table = "allot_order", fieldNamingPolicy = FieldNamingPolicy.LOWER_UNDERSCORE)
public class AllotOrderBO implements Serializable {

    @Serial
    private static final long serialVersionUID = 1L;

    /**
     * 调拨订单id
     */
    private Long allotOrderId;

    /**
     * 调出点仓库id
     */
    private Long outWarehouseId;

    /**
     * 调出点库存点类型
     * @see com.tmerclub.cloud.common.product.constant.StockPointType
     */
    private Integer outStockPointType;

    /**
     * 调入点仓库id
     */
    private Long inWarehouseId;

    /**
     * 调入点库存点类型
     * @see com.tmerclub.cloud.common.product.constant.StockPointType
     */
    private Integer inStockPointType;

    /**
     * 物流公司id
     */
    private Long dvyCompanyId;

    /**
     * 物流方式
     * @see com.tmerclub.cloud.common.order.constant.DeliveryType
     */
    private Integer dvyType;

    /**
     * 物流单号
     */
    private String dvyOrderNumber;

    /**
     * 状态
     */
    private Integer status;

    /**
     * 总调拨数量
     */
    private Integer totalAllotCount;

    /**
     * 备注
     */
    private String remark;

    /**
     * 店铺id
     */
    private Long shopId;

    /**
     * 系统类型
     * @see com.tmerclub.cloud.common.constant.SysTypeEnum
     */
    private Integer sysType;

    /**
     * 创建时间
     */
    private LocalDateTime createTime;

    /**
     * 更新时间
     */
    private LocalDateTime updateTime;

    public AllotOrderBO() {
    }

    public Long getAllotOrderId() {
        return allotOrderId;
    }

    public void setAllotOrderId(Long allotOrderId) {
        this.allotOrderId = allotOrderId;
    }

    public Long getOutWarehouseId() {
        return outWarehouseId;
    }

    public void setOutWarehouseId(Long outWarehouseId) {
        this.outWarehouseId = outWarehouseId;
    }

    public Integer getOutStockPointType() {
        return outStockPointType;
    }

    public void setOutStockPointType(Integer outStockPointType) {
        this.outStockPointType = outStockPointType;
    }

    public Long getInWarehouseId() {
        return inWarehouseId;
    }

    public void setInWarehouseId(Long inWarehouseId) {
        this.inWarehouseId = inWarehouseId;
    }

    public Integer getInStockPointType() {
        return inStockPointType;
    }

    public void setInStockPointType(Integer inStockPointType) {
        this.inStockPointType = inStockPointType;
    }

    public Long getDvyCompanyId() {
        return dvyCompanyId;
    }

    public void setDvyCompanyId(Long dvyCompanyId) {
        this.dvyCompanyId = dvyCompanyId;
    }

    public Integer getDvyType() {
        return dvyType;
    }

    public void setDvyType(Integer dvyType) {
        this.dvyType = dvyType;
    }

    public String getDvyOrderNumber() {
        return dvyOrderNumber;
    }

    public void setDvyOrderNumber(String dvyOrderNumber) {
        this.dvyOrderNumber = dvyOrderNumber;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Integer getTotalAllotCount() {
        return totalAllotCount;
    }

    public void setTotalAllotCount(Integer totalAllotCount) {
        this.totalAllotCount = totalAllotCount;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public Integer getSysType() {
        return sysType;
    }

    public void setSysType(Integer sysType) {
        this.sysType = sysType;
    }

    public LocalDateTime getCreateTime() {
        return createTime;
    }

    public void setCreateTime(LocalDateTime createTime) {
        this.createTime = createTime;
    }

    public LocalDateTime getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(LocalDateTime updateTime) {
        this.updateTime = updateTime;
    }

    @Override
    public String toString() {
        return "AllotOrderBO{" +
                "allotOrderId=" + allotOrderId +
                ", outWarehouseId=" + outWarehouseId +
                ", outStockPointType=" + outStockPointType +
                ", inWarehouseId=" + inWarehouseId +
                ", inStockPointType=" + inStockPointType +
                ", dvyCompanyId=" + dvyCompanyId +
                ", dvyType=" + dvyType +
                ", dvyOrderNumber='" + dvyOrderNumber + '\'' +
                ", status=" + status +
                ", totalAllotCount=" + totalAllotCount +
                ", remark='" + remark + '\'' +
                ", shopId=" + shopId +
                ", sysType=" + sysType +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                '}';
    }
}
