/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.user.bo.mongo;

import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.CompoundIndex;
import org.springframework.data.mongodb.core.index.CompoundIndexes;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;

/**
 * 用户积分记录mongodb
 * @author TRACK
 */
@Document("user_score_log")
@CompoundIndexes({
        @CompoundIndex(name = "idx_user_id", def = "{'userId':1}"),
        @CompoundIndex(name = "idx_biz_id", def = "{'bizId':1}")
})
public class MongoUserScoreLogBO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;

    @Id
    @Schema(description = "用户积分记录id", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long logId;

    @Schema(description = "创建时间", requiredMode = Schema.RequiredMode.REQUIRED)
    private Date createTime;

    @Schema(description = "更新时间")
    private Date updateTime;

    @Schema(description = "用户id", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long userId;

    @Schema(description = "1.购物 2.会员等级提升加积分 3.签到加积分 4.购物抵扣使用积分 5.积分过期", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer source;

    @Schema(description = "变动积分数额")
    private Long score;

    @Schema(description = "业务id")
    private Long bizId;

    @Schema(description = "出入类型 0=支出 1=收入", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer ioType;

    public Long getLogId() {
        return logId;
    }

    public void setLogId(Long logId) {
        this.logId = logId;
    }

    public Long getUserId() {
        return userId;
    }

    public void setUserId(Long userId) {
        this.userId = userId;
    }

    public Integer getSource() {
        return source;
    }

    public void setSource(Integer source) {
        this.source = source;
    }

    public Long getScore() {
        return score;
    }

    public void setScore(Long score) {
        this.score = score;
    }

    public Long getBizId() {
        return bizId;
    }

    public void setBizId(Long bizId) {
        this.bizId = bizId;
    }

    public Integer getIoType() {
        return ioType;
    }

    public void setIoType(Integer ioType) {
        this.ioType = ioType;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }

    @Override
    public String toString() {
        return "MongoUserScoreLogBO{" +
                "logId=" + logId +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                ", userId=" + userId +
                ", source=" + source +
                ", score=" + score +
                ", bizId=" + bizId +
                ", ioType=" + ioType +
                '}';
    }
}
